function Kinetics1D
%============================================================
%============================================================
% Program for neutron kinetics in a 1D slab nuclear reactor
% Walter Ambrosini, March 26th, 2008
%============================================================
%============================================================
clc, clear;
%============================c
% Input data are assigned
%============================c
% NB: Lengths are expressed in cm
%----------------------------c
% Left reflector parameters
%----------------------------c
nrefl = 20 ;
dxrefl = 20. ;
%
dxrl = dxrefl / nrefl ;
n1 = nrefl ;
%
  for i=1:n1
  dx(i) = dxrl ;
  xm(i) = ( i - 0.5 ) * dxrl ;
%
  v1(i) = 1.25e07 ;
  d1(i) = 1.634 ;
  sa1(i) = 0.00266  ;
  ansf1(i) = 0. ;
  ss1to2(i) = 0.0276 ; 
  v2(i) = 2.5e05 ;
  d2(i) = 0.264 ;
  sa2(i) = 0.0494 ;
  ansf2(i) = 0. ;
%  
  sou1(i) = 0. ;
  sou2(i) = 0. ;
%
  end
%  
x1 = xm(n1) + 0.5 * dxrl ;
%
%----------------------------------c
% Multiplicating medium parameters
%----------------------------------c
nmult = 200 ;
dxmult = 200. ;
%
dxmu = dxmult / nmult;
n2 = nrefl + nmult;
%
  for i=n1+1:n2
  dx(i) = dxmu ;
  xm(i) = x1 + ( i - n1 - 0.5 ) * dxmu ;
%
  v1(i) = 1.25e07 ;
  d1(i) = 1.426 ;
  sa1(i) = 0.01099  ;
  ansf1(i) = 0.0075 ;
  ss1to2(i) = 0.0176 ; 
  v2(i) = 2.5e05 ;
  d2(i) = 0.350 ;
  sa2(i) = 0.09926 ;
  ansf2(i) = 0.1378 ;
%  
  sou1(i) = 0. ;
  sou2(i) = 0. ;
%
  end
%  
x2 = xm(n2) + 0.5 * dxmu ;
%
%----------------------------c
% Right reflector parameters
%----------------------------c
nrefr = 20 ;
dxrefr = 20. ;
%
dxrr = dxrefr / nrefr ;
n3 = nrefl + nmult + nrefr ;
ntot = n3 ;
%
  for i=n2+1:n3
  dx(i) = dxrr ;
  xm(i) = x2 + ( i - n2 - 0.5 ) * dxrr ;
%
  v1(i) = 1.25e07 ;
  d1(i) = 1.634 ;
  sa1(i) = 0.00266  ;
  ansf1(i) = 0. ;
  ss1to2(i) = 0.0276 ; 
  v2(i) = 2.5e05 ;
  d2(i) = 0.264 ;
  sa2(i) = 0.0494 ;
  ansf2(i) = 0. ;
%  
  sou1(i) = 0. ;
  sou2(i) = 0. ;
%
  end
%  
x3 = xm(n3) + 0.5 * dxrr ;
%
alef(1:n3) = 0. ;
bet(1:n3) = 0. ;
%
%------------------------------------c
% Parameters for kinetic calculations
%------------------------------------c
%
  alambd(1) = 0.0127 ;   
  alambd(2) = 0.0317 ;   
  alambd(3) = 0.115 ;   
  alambd(4) = 0.311 ;   
  alambd(5) = 1.4 ;   
  alambd(6) = 3.87 ;   
%
  beta(1) = 0.000247 ;   
  beta(2) = 0.0013845 ;   
  beta(3) = 0.001222 ;   
  beta(4) = 0.0026455 ;   
  beta(5) = 0.000832 ;   
  beta(6) = 0.000169 ;   
%
   bttot = 0. ;
   for id = 1:6
       bttot = bttot + beta(id) ;
   end
%   
disp('Value of beta = '); disp(bttot);
%
%---------------------------------c
% Averaged D/dx coefficients
%---------------------------------c
%
  d1ovdx(1) = 2. * d1(1) / dx(1) ;   
  d2ovdx(1) = 2. * d2(1) / dx(1) ;   
%  
  for i=2:n3
  aux1 = d1(i-1) / dx(i-1) ;
  aux2 = d1(i) / dx(i) ;
  denom = aux1 + aux2 ;
  d1ovdx(i) = 2. * aux1 * aux2 / denom ;   
%  
  aux1 = d2(i-1) / dx(i-1) ;
  aux2 = d2(i) / dx(i) ;
  denom = aux1 + aux2 ;
  d2ovdx(i) = 2. * aux1 * aux2 / denom ;   
  end
%
  d1ovdx(n3+1) = 2. * d1(n3) / dx(n3) ;   
  d2ovdx(n3+1) = 2. * d2(n3) / dx(n3) ;  
%  
%-------------- Search for criticality  -----
disp('Criticality Search ');
%--------------------------------------------  
%
% Interactive input for control rods
%
disp('Number of nodes in the left reflector = '); disp(n1);
disp('Number of nodes in the multiplicating medium = '); disp(n2-n1);
disp('Number of nodes in the right reflector = '); disp(n3-n2);
%
  ncrod=input('Assign the number of control rods ');
    if(ncrod>0) 
       for ic=1:ncrod
       disp('Control rod number = ');
       disp(ic);
       ncr=input('Enter the node index of the control rod ');
       disp('Present values of Sigma1 and Sigma2 = ');
       disp(sa1(ncr));
       disp(sa2(ncr));
       sa1(ncr)=input('New value of Sigma1 ');
       sa2(ncr)=input('New value of Sigma2 ');
       end
    end;
%  
%===============================
% Criticality search
%===============================
%
% Initial value of keff and fission source
%
akeff = 1.;
fissou(1:n1) = 0. ;
fissou(n1+1:n2) = 1.;
fissou(n2+1:n3) = 0.;
%
% Outer Iteration loop
%  
  for iter=1:10000
%--------------------------------------------------------------------c      
% construction of the vectors a, b, c and d for the 1st energy group
%--------------------------------------------------------------------c
    for i=1:n3
    a(i) = - d1ovdx(i) ;
    b(i) = d1ovdx(i) + d1ovdx(i+1) + ( sa1(i) + ss1to2(i) ) * dx(i) ;
    c(i) = - d1ovdx(i+1) ;
    d(i) = fissou(i) / akeff ;
    end
%
%  Solution of the TDM system for the 1st energy group 
%
  t(1:ntot) = 0. ;
%
  [t]=tdma(a,b,c,d,alef,bet,ntot);
%
    for i=1:n3
    phi1(i) = t(i) ;
    end
%--------------------------------------------------------------------c      
% construction of the vectors a, b, c and d for the 2nd energy group
%--------------------------------------------------------------------c
    for i=1:n3
    a(i) = - d2ovdx(i) ;
    b(i) = d2ovdx(i) + d2ovdx(i+1) + sa2(i) * dx(i) ;
    c(i) = - d2ovdx(i+1) ;
    d(i) = ss1to2(i) * dx(i) * phi1(i) ;
    end
%
%  Solution of the TDM system for the 1st energy group 
%
  t(1:ntot) = 0. ;
%
  [t]=tdma(a,b,c,d,alef,bet,ntot);
%
    for i=1:n3
    phi2(i) = t(i) ;
    end
%-------------------------------------
%  Updating keff and Fission source
%-------------------------------------
  oldkef = akeff ;
  sumold = 0. ;
  sumfis = 0. ;
  amxph1 = -1. ;
  amxph2 = - 1. ;
%    
    for i=1:n3
    oldfis(i) = fissou(i) ;
    fissou(i) = ( ansf1(i) * phi1(i) + ansf2(i) * phi2(i) ) * dx(i) ;
%  
    sumold = sumold + oldfis(i) ;
    sumfis = sumfis + fissou(i) ;
%    
      if (amxph1<phi1(i)) 
          amxph1 = phi1(i) ;
      end;
      if (amxph2<phi2(i)) 
        amxph2 = phi2(i) ;
      end;
%
    amxphi = amxph1 ;
      if (amxphi<amxph2) 
         amxphi = amxph2 ;
      end;
    end
%
  akeff = oldkef * sumfis / sumold ;
%  disp('Iteration number and keff = '),iter,akeff;
  
  chk = num2str(akeff,'%10.5f');
  chiter = num2str(iter,'%10.0f');
%
  erkeff = abs( akeff - oldkef ) ;
  if (erkeff<1.e-06),break,end
% 
  end
%------------------------------------------------
%---------  end of criticality search loop ------
%------------------------------------------------
% A plot of fluxes is generated
%
  plot(xm(1:n3),phi1(1:n3),'b.',xm(1:n3),phi2(1:n3),'r.')
    grid
    xlabel('x [cm]')
    ylabel('Normalised Neutron Flux')
    axis([0., x3, 0., 2*amxphi])
    title(['Iter = ',chiter,'     Keff = ',chk]) 
    h = legend('Fast Neutron Flux','Thermal Neutron Flux',1);
%    
%---------------------------------------------------------------
% Preparing the transient calculation
%---------------------------------------------------------------
%
% Interactive input on starting criticality level 
%
  aknew=input('Assign the new Keff ');
%
% The fission cross sections are reassigned
%
  fact = aknew / akeff ;
%
  sumfis = 0. ;
       for i=1:ntot
       ansf1(i) = ansf1(i) * fact ;
       ansf2(i) = ansf2(i) * fact ;
       fissou(i) = fissou(i) * fact ;
       sumfis = sumfis + fissou(i) ;
       end
%    
%---------------------------------------------------------------
% Transient calculation
%---------------------------------------------------------------
%
disp('Transient Calculation ');
%
%------------------------------------------------------------------------
%  Level of implicit calculation
%------------------------------------------------------------------------
    theta=input('Enter theta [0.=Explicit; 0.5=C-N; 1.= Implicit] ');
%    
    cmptht = 1. - theta ;
    cmpbet = 1. - bttot ;
%  
% Interactive input on localised sources
%
disp('Number of nodes in the left reflector = '); disp(n1);
disp('Number of nodes in the multiplicating medium = '); disp(n2-n1);
disp('Number of nodes in the right reflector = '); disp(n3-n2);
%
  nsour=input('Assign the number of localised sources ');
    if(nsour>0) 
       for is=1:nsour
       disp('Localised source number = ');
       disp(is);
       nsr=input('Enter the node index of the localised sources ');
       disp('Present values of Sou1 and Sou2 = ');
       disp(sou1(nsr));
       disp(sou2(nsr));
       sou1(nsr)=input('New value of Sou1 ');
       sou2(nsr)=input('New value of Sou2 ');
       end
    end;
%  
%  Assigning the delayed neutron precursors concentrations at steady-state
%  
      for id = 1:6
          for i = 1:ntot
          cd(id,i) = beta(id) * fissou(i) / alambd(id) ;
          end
      end
%
time = 0. ;
tread = 0. ;
%--------------------------------
%  Time advancement loop - start
%--------------------------------
%
for itime=1:100000
%
% Assigning the old fluxes and precursor concentrations
%
   for i = 1:ntot
     oldph1(i) = phi1(i) ;
     oldph2(i) = phi2(i) ;
     for id = 1:6
         oldcd(id,i) = cd(id,i) ;
     end
   end
%  
% Interactive input for control rod movement
%
    if(time>=tread)        
      disp('Number of nodes in the left reflector = '); disp(n1);
      disp('Number of nodes in the multiplicating medium = '); disp(n2-n1);
      disp('Number of nodes in the right reflector = '); disp(n3-n2);
%
      ncrod=input('Assign the number of control rods ');
       if(ncrod>0) 
          for ic=1:ncrod
          disp('Control rod number = ');
          disp(ic);
          ncr=input('Enter the node index of the control rod ');
          disp('Present values of Sigma1 and Sigma2 = ');
          disp(sa1(ncr));
          disp(sa2(ncr));
          sa1(ncr)=input('New value of Sigma1 ');
          sa2(ncr)=input('New value of Sigma2 ');
          end
       end;
       tread=input('Assign the new break time [s] ');      
       dt=input('Assign the time step [s] ');      
    end;
%
    time = time + dt ;
    tv(itime) = time ;
%    disp(time);
%
% Iteration on the implict terms on the RHS
%    
   for iter = 1:1000
%    disp(iter);
%
%--------------------------------------------------------------------c      
% construction of the vectors a, b, c and d for the 1st energy group
%--------------------------------------------------------------------c
         for i=1:ntot
%             
         a(i) = - theta * d1ovdx(i) ;
%    
         aux1 = dx(i) / ( v1(i) * dt ) ;
         aux2 = d1ovdx(i) + d1ovdx(i+1) ;
         aux3 = ( sa1(i) + ss1to2(i) - cmpbet * ansf1(i) ) * dx(i) ;
         b(i) = aux1 + theta * ( aux2 + aux3 ) ;
%    
         c(i) = - theta * d1ovdx(i+1) ;
%
         aux1 = theta * cmpbet * ansf2(i) * phi2(i) * dx(i) ;
         aux2 = 0. ;
            for id = 1:6
                aux2 = aux2 + alambd(id) * cd(id,i) ;
            end
         aux2 = aux2 * theta * dx(i) ;
%            
         aux3 = dx(i) / ( v1(i) * dt ) * oldph1(i) ;
% 
         aux4 = - ( d1ovdx(i) + d1ovdx(i+1) ) * oldph1(i) ;
           if(i>1) 
             aux4 = aux4 + d1ovdx(i) * oldph1(i-1) ;
           end;
           if(i<ntot)
             aux4 = aux4 + d1ovdx(i+1) * oldph1(i+1) ;
           end;
         aux4 = aux4 - ( sa1(i) + ss1to2(i) ) * dx(i) * oldph1(i) ;
         auxfis = ansf1(i) * oldph1(i) + ansf2(i) * oldph2(i) ;
         aux4 = aux4 + cmpbet * auxfis * dx(i) ;
            for id = 1:6
                aux4 = aux4 + alambd(id) * oldcd(id,i) * dx(i);
            end
         aux4 = aux4 * cmptht ;
%
         aux5 = sou1(i) * dx(i) ;
%         
         d(i) = aux1 + aux2 + aux3 + aux4 + aux5 ;
         end
%
%  Solution of the TDM system for the 1st energy group 
%
    t(1:ntot) = 0. ;
%
    [t]=tdma(a,b,c,d,alef,bet,ntot);
%
         for i=1:ntot
         phi1(i) = t(i) ;
         end
%--------------------------------------------------------------------c      
% construction of the vectors a, b, c and d for the 2nd energy group
%--------------------------------------------------------------------c
         for i=1:ntot
         a(i) = - theta * d2ovdx(i) ;
%    
         aux1 = dx(i) / ( v2(i) * dt ) ;
         aux2 = d2ovdx(i) + d2ovdx(i+1) ;
         aux3 = sa2(i) * dx(i) ;
         b(i) = aux1 + theta * ( aux2 + aux3 ) ;
%    
         c(i) = - theta * d2ovdx(i+1) ;
%
         aux1 = dx(i) / ( v2(i) * dt ) * oldph2(i) ;
         aux2 = theta * ss1to2(i) * phi1(i) * dx(i) ;            
% 
         aux3 = - ( d2ovdx(i) + d2ovdx(i+1) ) * oldph2(i) ;
         if(i>1) 
             aux3 = aux3 + d2ovdx(i) * oldph2(i-1) ;
         end;
         if(i<ntot)
             aux3 = aux3 + d2ovdx(i+1) * oldph2(i+1) ;
         end;
         aux3 = aux3 - ( sa2(i)*oldph2(i) - ss1to2(i)*oldph1(i)) * dx(i)  ;
         aux3 = aux3 * cmptht ;
%
         aux4 = sou2(i) * dx(i) ;
%         
         d(i) = aux1 + aux2 + aux3 + aux4 ;
         end
%
%  Solution of the TDM system for the 1st energy group 
%
    t(1:ntot) = 0. ;
%
    [t]=tdma(a,b,c,d,alef,bet,ntot);
%
        for i=1:ntot
        phi2(i) = t(i) ;
        end
%------------------------------------------------------------
%  Advancing the precursors' concentrations
%------------------------------------------------------------
%       
   fisnew = 0. ;
%       
       for i = 1:ntot
           auxfo = ansf1(i) * oldph1(i) + ansf2(i) * oldph2(i) ;
           auxfn = ansf1(i) * phi1(i) + ansf2(i) * phi2(i) ;
           fisnew = fisnew + auxfn * dx(i) ;
           for id =1:6
               expldt = exp ( - alambd(id) * dt ) ;
               cmpexp = 1. - expldt ;
               aux1 = oldcd(id,i) * expldt ;
               aux2 = auxfo * cmptht + auxfn * theta ;
               aux2 = aux2 * cmpexp * beta(id) / alambd(id) ;
               cd(id,i) = aux1 + aux2 ;
           end
       end
   fv(itime) = fisnew ;
%
%  Iteration stopping criterion
% 
       if(iter==1)
           fisold = fisnew;
       else
           errel = abs (fisnew - fisold ) / fisnew ;
           if (errel<1.e-05), break,end
%           disp(errel);
           fisold = fisnew ;
       end;
       
   end
%
%  Calculating the reactor period
%
   if(itime>1)
      recper = ( fv(itime) - fv(itime-1) ) / ( fv(itime) * dt ) ;
   else
      recper = ( fv(itime) - sumfis ) / ( fv(itime) * dt ) ; 
   end;
   per = 1. / recper ;
   pv(itime) = per ;
   rpv(itime) = recper ;
%       
% Plots of fluxes and of the overall nuetron source is generated
%
  chtime = num2str(time,'%10.5f');
  chiter = num2str(iter,'%10.0f');
  chfv = num2str(fv(itime),'%10.3f');
  chpv = num2str(pv(itime),'%10.3f');
%
   subplot (1,3,1);
   plot(xm(1:n3),phi1(1:n3),'b.',xm(1:n3),phi2(1:n3),'r.')
    grid
    xlabel('x [cm]')
    ylabel('Normalised Neutron Flux')
    axis([0., x3,0.,2*amxphi])
    title(['Time = ',chtime,' s , Iter =',chiter]) 
    h = legend('Fast Neutron Flux','Thermal Neutron Flux',1);
%
    subplot (1,3,2);
   plot(0.,sumfis,'r.',tv(1:itime),fv(1:itime),'b.')
    grid
    xlabel('Time [s]')
    ylabel('Normalised Fission Source')
    axis([0., 50.,0.,1000.])
    title(['Time = ',chtime,' s , Fission Source = ',chfv]) 
%
    subplot (1,3,3);
   plot(tv(1:itime),pv(1:itime),'b.')
    grid
    xlabel('Time [s]')
    ylabel('Reactor Period [s]')
    axis([0., 50.,-100.,100.])
    title(['Time = ',chtime,' s , Reactor Period = ',chpv,' [s]' ]) 
    pause (0.0001);
%    
%--------------------------------
%  Time advancement loop - end
%--------------------------------
end 
%----------------------------
%  End of Program
%----------------------------
end
%
%-------------------------------------------%
% Solution of the tridiagonal matrix system %
%-------------------------------------------%
function [t]=tdma(a,b,c,d,alef,bet,ntot);
%
alef(1:ntot)=0.;
bet(1:ntot)=0.;
t(1:ntot)=0.;
%
ub=1.d00/b(1);
alef(1)=c(1)*ub;
bet(1)=d(1)*ub;
%
    for i=2:ntot
      l=i-1;
      qz=b(i)-a(i)*alef(l);
      uqz=1.d00/qz;
      alef(i)=c(i)*uqz;
      bet(i)=(d(i)-a(i)*bet(l))*uqz;
    end
%    
nm1=ntot-1;
t(ntot)=bet(ntot);
    for i=1:nm1
      ii=ntot-i;
      l=ii+1;
      t(ii)=bet(ii)-alef(ii)*t(l);
    end
%
end 
